#pragma once
#define LGFX_USE_V1
#include <LovyanGFX.hpp>

class LGFX : public lgfx::LGFX_Device {
  lgfx::Panel_ST7789 _panel_instance; // 表示パネルインスタンス
  lgfx::Bus_SPI _bus_instance;        // SPIバスインスタンス
  lgfx::Light_PWM _light_instance;    // バックライト制御インスタンス

  public:
    LGFX(void) {
      { // 表示パネル制御設定
        auto cfg = _panel_instance.config();  // 表示パネル設定構造体取得

        cfg.pin_cs = 10;          // CS(Disable = -1)
        cfg.pin_rst = 1;          // RST(Disable = -1)
        cfg.pin_busy = -1;        // BUSY(Disable = -1)
        cfg.panel_width = 170;    // 表示幅
        cfg.panel_height = 320;   // 表示高
        cfg.offset_x = 35;        // X方向オフセット
        cfg.offset_y = 0;         // Y方向オフセット
        cfg.offset_rotation = 0;  // 回転方向の値のオフセット 0～7（4～7は上下反転）
        cfg.dummy_read_pixel = 8; // ピクセル読取前のダミーリードビット数
        cfg.dummy_read_bits  = 1; // ピクセル以外のデータ読取前のダミーリードビット数
        cfg.readable = true;      // データ読取が可能なときtrueに設定
        cfg.invert = true;       // パネルの明暗が反転するとき trueに設定
        cfg.rgb_order = false;    // パネルの赤と青が入れ替わるとき trueに設定
        cfg.dlen_16bit = false;   // 16bitパラレルやSPIでデータ長を16bit単位で送信するパネルのときtrueに設定
        cfg.bus_shared = false;   // SDカードとバスを共有しているときtrueに設定（drawJpgFileなどでバス制御）

        _panel_instance.config(cfg);
      }

      {  // バス制御設定
        auto cfg = _bus_instance.config();  // バス設定構造体取得

        // SPIバス設定
        cfg.spi_host = SPI3_HOST;           // SPI2_HOST(VSPI) or SPI3_HOST(HSPI)
        cfg.spi_mode = 0;                   // SPI通信モード設定（0～3）
        cfg.freq_write = 40000000;          // 送信SPIクロック（最大80MHz、80MHzを整数で割った値）
        cfg.freq_read = 16000000;           // 受信SPIクロック
        cfg.spi_3wire = true;               // 受信をMOSIピンで行うときtrueに設定
        cfg.use_lock = true;                // トランザクションロック使用するときtrueに設定
        cfg.dma_channel = SPI_DMA_CH_AUTO;  // DMAチャンネル設定 (DMA不使用 = 0、1ch = 1、2ch = 2、推奨⇒自動設定 = SPI_DMA_CH_AUTO）
        cfg.pin_sclk = 12;                  // SCLK
        cfg.pin_mosi = 13;                  // MOSI
        cfg.pin_miso = -1;                  // MISO(Disable = -1)
        cfg.pin_dc = 11;                    // DC(Disable = -1)

        _bus_instance.config(cfg);
        _panel_instance.setBus(&_bus_instance);  // バスをパネルにセット
      }

      {  // バックライト制御設定
        auto cfg = _light_instance.config();  // バックライト設定構造体取得

        cfg.pin_bl = 14;      // ピン番号
        cfg.invert = false;   // 輝度を反転させるときtrueに設定
        cfg.freq = 44100;     // PWM周波数
        cfg.pwm_channel = 7;  // PWMのチャンネル番号

        _light_instance.config(cfg);
        _panel_instance.setLight(&_light_instance);  // バックライトをパネルにセット
      }

      setPanel(&_panel_instance);  // パネルをセット
    }
};
